@@ -10,9 +10,11 @@ import warnings
 from chainladder.core.display import TriangleDisplay
 from chainladder.core.dunders import TriangleDunders
 from chainladder.core.pandas import TrianglePandas
-from chainladder.core.slice import TriangleSlicer
+from chainladder.core.slice import TriangleSlicer, VirtualColumns
 from chainladder.core.io import TriangleIO
 from chainladder.core.common import Common
+from chainladder import AUTO_SPARSE, ULT_VAL
+from chainladder.utils.utility_functions import num_to_nan, concat
 
 
 class TriangleBase(
@@ -31,19 +33,17 @@ class TriangleBase(
         development_format=None,
         cumulative=None,
         array_backend=None,
+        pattern=False,
         *args,
         **kwargs
     ):
-        from chainladder.utils.utility_functions import num_to_nan
-        from chainladder import AUTO_SPARSE
-
         # Allow Empty Triangle so that we can piece it together programatically
         if data is None:
             return
 
         # Check whether all columns are unique and numeric
         check = data[columns].dtypes
-        check = [check] if check.__class__.__name__ == "dtype" else check.to_list()
+        check = [check] if isinstance(check, np.dtype) else check.to_list()
         columns = [columns] if type(columns) is not list else columns
         if "object" in check:
             raise TypeError("column attribute must be numeric.")
@@ -63,8 +63,27 @@ class TriangleBase(
             from chainladder import ARRAY_BACKEND
 
             array_backend = ARRAY_BACKEND
-
+        if (
+            development
+            and len(development) == 1
+            and data[development[0]].dtype == "<M8[ns]"
+        ):
+            u = data[data[development[0]] == ULT_VAL].copy()
+            if len(u) > 0 and len(u) != len(data):
+                u = TriangleBase(
+                    u,
+                    origin=origin,
+                    development=development,
+                    columns=columns,
+                    index=index,
+                )
+                data = data[data[development[0]] != ULT_VAL]
+            else:
+                u = None
+        else:
+            u = None
         # Initialize origin and its grain
+        origin = development if origin is None else origin
         origin_date = TriangleBase._to_datetime(data, origin, format=origin_format)
         self.origin_grain = TriangleBase._get_grain(origin_date)
         origin_date = (
@@ -75,7 +94,8 @@ class TriangleBase(
 
         # Initialize development and its grain
         m_cnt = {"Y": 12, "Q": 3, "M": 1}
-        if development:
+        has_dev = development and len(np.unique(data[development])) > 1
+        if has_dev:
             development_date = TriangleBase._to_datetime(
                 data, development, period_end=True, format=development_format
             )
@@ -90,7 +110,7 @@ class TriangleBase(
 
         # Summarize dataframe to the level specified in axes
         key_gr = [origin_date, development_date] + [
-            data[item] for item in self._flatten(index)
+            data[item] for item in ([] if not index else index)
         ]
         data_agg = data[columns].groupby(key_gr).sum().reset_index().fillna(0)
         if not index:
@@ -111,6 +131,7 @@ class TriangleBase(
                 date_axes["origin"], date_axes["development"]
             ).unique()
         )
+
         orig_unique = np.sort(date_axes["origin"].unique())
         kdims = data_agg[index].drop_duplicates().reset_index(drop=True).reset_index()
 
@@ -153,34 +174,58 @@ class TriangleBase(
         self.array_backend = "sparse"
         self.values = num_to_nan(
             sp(
-                coords.T,
+                coords.T.astype('int64'),
                 amts,
                 prune=True,
+                has_duplicates=False,
+                sorted=True,
                 shape=(
                     len(kdims),
                     len(columns),
                     len(orig_unique),
-                    len(dev_lag_unique) if development else 1,
+                    len(dev_lag_unique) if has_dev else 1,
                 ),
             )
         )
 
         # Set all axis values
+        self.valuation_date = data_agg["development"].max()
         self.kdims = kdims.drop("index", 1).values
         self.odims = orig_unique
-        self.ddims = dev_lag_unique if development else dev_lag[0:1].values
+        self.ddims = dev_lag_unique if has_dev else dev_lag[0:1].values
         self.ddims = self.ddims * (m_cnt[self.development_grain])
+        if development and not has_dev:
+            self.ddims = pd.DatetimeIndex(TriangleBase._to_datetime(
+                data, development, period_end=True, format=development_format)[0:1])
+            self.valuation_date = self.ddims[0]
         self.vdims = np.array(columns)
 
         # Set remaining triangle properties
         self.key_labels = index
         self.is_cumulative = cumulative
-        self.valuation_date = data_agg["development"].max()
+        self.virtual_columns = VirtualColumns(self)
+        self.is_pattern = pattern
         if not AUTO_SPARSE or array_backend == "cupy":
             self.set_backend(array_backend, inplace=True)
         else:
             self = self._auto_sparse()
         self._set_slicers()
+        if self.is_pattern:
+            obj = self.dropna()
+            self.odims = obj.odims
+            self.ddims = obj.ddims
+            self.values = obj.values
+        if u:
+            obj = concat((self.dev_to_val().iloc[..., : len(u.odims), :], u), -1)
+            obj = obj.val_to_dev()
+            self.odims = obj.odims
+            self.ddims = obj.ddims
+            self.values = obj.values
+            self.valuation_date = pd.Timestamp(ULT_VAL)
+
+    @property
+    def shape(self):
+        return self.values.shape
 
     def _len_check(self, x, y):
         if len(x) != len(y):
@@ -289,7 +334,7 @@ class TriangleBase(
             target = target_field.map(arr)
         if period_end:
             target = target.dt.to_period(
-                TriangleBase._get_grain(target)
+                "M" if len(target.unique()) == 1 else TriangleBase._get_grain(target),
             ).dt.to_timestamp(how="e")
         return target
 
@@ -312,9 +357,7 @@ class TriangleBase(
 
     @staticmethod
     def _get_grain(array):
-        months = set(array.dt.month)
-        grain = {**{1: "Y", 4: "Q"}, **{item: "M" for item in range(5, 13)}}
-        return grain[len(months)]
+        return {1: "Y", 4: "Q"}.get(len(set(array.dt.month)), "M")
 
     @staticmethod
     def _cartesian_product(*arrays):
@@ -328,17 +371,6 @@ class TriangleBase(
         arr = arr.reshape(-1, len(arrays))
         return arr
 
-    def _flatten(self, *args):
-        return_list = []
-        for item in args:
-            return_list = return_list + item if item else []
-        return return_list
-
-    def num_to_nan(self):
-        from chainladder.utils.utility_functions import num_to_nan
-
-        self.values = num_to_nan(self.values)
-
     def get_array_module(self, arr=None):
         backend = (
             self.array_backend
@@ -357,15 +389,55 @@ class TriangleBase(
 
         if not AUTO_SPARSE:
             return self
-        n = np.prod(self.shape)
+        n = np.prod(list(self.shape) + [8 / 1e6])
         if (
             self.array_backend == "numpy"
-            and n / 8e6 > 30
-            and np.isnan(self.values).sum() / n < 0.2
+            and n > 30
+            and 1 - np.isnan(self.values).sum() / n * (8 / 1e6) < 0.2
         ):
             self.set_backend("sparse", inplace=True)
         if self.array_backend == "sparse" and not (
-            self.values.density < 0.2 and n / 8e6 > 30
+            self.values.density < 0.2 and n > 30
         ):
             self.set_backend("numpy", inplace=True)
         return self
+
+    def copy(self):
+        X = TriangleBase()
+        X.__dict__.update(vars(self))
+        X._set_slicers()
+        return X
+
+    @property
+    def valuation(self):
+        ddims = self.ddims
+        if self.is_val_tri:
+            out = pd.DataFrame(np.repeat(self.ddims.values[None], len(self.odims), 0))
+            return pd.DatetimeIndex(out.unstack().values)
+        ddim_arr = ddims - ddims[0]
+        origin = np.minimum(self.odims, np.datetime64(self.valuation_date))
+        val_array = origin.astype("datetime64[M]") + np.timedelta64(ddims[0], "M")
+        val_array = val_array.astype("datetime64[ns]") - np.timedelta64(1, "ns")
+        val_array = val_array[:, None]
+        s = slice(None, -1) if ddims[-1] == 9999 else slice(None, None)
+        val_array = (
+            val_array.astype("datetime64[M]") + ddim_arr[s][None, :] + 1
+        ).astype("datetime64[ns]") - np.timedelta64(1, "ns")
+        if ddims[-1] == 9999:
+            ult = np.repeat(np.datetime64(ULT_VAL), val_array.shape[0])[:, None]
+            val_array = np.concatenate((val_array, ult,), axis=1,)
+        return pd.DatetimeIndex(val_array.reshape(1, -1, order="F")[0])
+
+
+def is_chainladder(estimator):
+    """Return True if the given estimator is a chainladder based method.
+    Parameters
+    ----------
+    estimator : object
+        Estimator object to test.
+    Returns
+    -------
+    out : bool
+        True if estimator is a chainladder based method and False otherwise.
+    """
+    return getattr(estimator, "_estimator_type", None) == "chainladder"
