@@ -4,6 +4,7 @@
 import pandas as pd
 import numpy as np
 import warnings
+from chainladder.utils.utility_functions import num_to_nan
 
 
 class TriangleDunders:
@@ -19,10 +20,9 @@ class TriangleDunders:
             obj, other = self._prep_index(obj, other)
             obj, other = self._prep_columns(obj, other)
             xp = obj.get_array_module()
-            a, b = self.shape[-2:], other.shape[-2:]
-            is_broadcastable = (
-                a[0] == 1 or b[0] == 1 or np.all(other.odims == obj.odims)
-            ) and (a[1] == 1 or b[1] == 1 or np.all(other.ddims == obj.ddims))
+            is_broadcastable = all(
+                (m == n) or (m == 1) or (n == 1)
+                for m, n in zip(obj.shape[::-1], other.shape[::-1]))
             if is_broadcastable:
                 if len(other.odims) == 1 and len(obj.odims) > 1:
                     other.odims = obj.odims
@@ -88,19 +88,14 @@ class TriangleDunders:
 
     def _arithmetic_cleanup(self, obj, other):
         """ Common functionality AFTER arithmetic operations """
-        from chainladder.utils.utility_functions import num_to_nan
-
         obj.values = obj.values * obj.get_array_module().nan_to_num(obj.nan_triangle)
-        obj.num_to_nan()
+        obj.values = num_to_nan(obj.values)
         return obj
 
     def _compatibility_check(self, x, y):
-        from chainladder import ARRAY_PRIORITY
+        from chainladder.utils.utility_functions import set_common_backend
 
-        backend = ARRAY_PRIORITY[
-            min([ARRAY_PRIORITY.index(x) for x in [x.array_backend, y.array_backend]])
-        ]
-        x, y = x.set_backend(backend), y.set_backend(backend)
+        x, y = set_common_backend([x, y])
         if (
             x.origin_grain != y.origin_grain
             or x.development_grain != y.development_grain
@@ -133,6 +128,15 @@ class TriangleDunders:
                 y = apply_axis(common, y, x) if len(y.index) > 1 else y
             else:
                 raise ValueError("Triangle arithmetic along index is ambiguous.")
+        if (
+            x.key_labels == y.key_labels
+            and len(x) == len(y)
+            and len(y) > 1
+            and not np.all(x.index == y.index)
+        ):
+            y = y.loc[x.index]
+        if len(x) == 1 and len(y) > 1:
+            x.kdims = y.kdims
         return x, y
 
     def _prep_columns(self, x, y):
@@ -201,7 +205,7 @@ class TriangleDunders:
     def __mul__(self, other):
         obj, other = self._validate_arithmetic(other)
         xp = obj.get_array_module()
-        obj.values = xp.nan_to_num(obj.values) * other
+        obj.values = obj.values * other
         return self._arithmetic_cleanup(obj, other)
 
     def __rmul__(self, other):
@@ -222,13 +226,13 @@ class TriangleDunders:
     def __truediv__(self, other):
         obj, other = self._validate_arithmetic(other)
         xp = obj.get_array_module()
-        obj.values = xp.nan_to_num(obj.values) / other
+        obj.values = obj.values / other
         return self._arithmetic_cleanup(obj, other)
 
     def __rtruediv__(self, other):
         obj = self.copy()
         obj.values = other / self.values
-        obj.num_to_nan()
+        obj.values = num_to_nan(obj.values)
         return obj
 
     def __eq__(self, other):
@@ -263,20 +267,7 @@ class TriangleDunders:
         obj.values = xp.nan_to_num(obj.values) < xp.nan_to_num(value)
         return obj
 
-    def minimum(self, other):
-        obj = self.copy()
-        xp = self.get_array_module()
-        obj.values = xp.minimum(self.values, other)
-        return obj
-
-    def maximum(self, other):
-        obj = self.copy()
-        xp = self.get_array_module()
-        obj.values = xp.maximum(self.values, other)
-        return obj
-
-    def sqrt(self):
-        obj = self.copy()
-        xp = self.get_array_module()
-        obj.values = xp.sqrt(self.values)
-        return obj
+    def __contains__(self, value):
+        if self.__dict__.get(value, None) is None:
+            return False
+        return True
