@@ -31,10 +31,13 @@ class _LocBase:
         obj.vdims = obj.vdims[c_idx]
         obj.odims, obj.ddims = obj.odims[o_idx], obj.ddims[d_idx]
         obj.iloc, obj.loc = Ilocation(obj), Location(obj)
+        obj.valuation_date = np.minimum(obj.valuation.max(), obj.valuation_date)
         return obj
 
     @staticmethod
     def _contig_slice(arr):
+        if type(arr) is pd.Series:
+            arr = arr[arr].index.tolist()
         if type(arr) is slice:
             return arr
         if type(arr) in [int, np.int64, np.int32]:
@@ -42,6 +45,8 @@ class _LocBase:
         if len(arr) == 1:
             return slice(arr[0], arr[0] + 1)
         diff = np.diff(arr)
+        if len(diff) == 0:
+            raise ValueError("Slice returns empty Triangle")
         if max(diff) == min(diff):
             step = max(diff)
         else:
@@ -64,45 +69,85 @@ class _LocBase:
         )
         if self.obj.array_backend == "sparse":
             self.obj.set_backend("numpy", inplace=True)
-            print(self.obj.values)
-            self.obj.values.__setitem__(normalize_index(key, self.obj.shape), values)
+            self.obj.values.__setitem__(self._normalize_index(key), values)
             self.obj.set_backend("sparse", inplace=True)
         else:
-            self.obj.values.__setitem__(normalize_index(key, self.obj.shape), values)
+            self.obj.values.__setitem__(self._normalize_index(key), values)
+
+    def _normalize_index(self, key):
+        key = normalize_index(key, self.obj.shape)
+        l = []
+        for n, i in enumerate(key):
+            if type(i) is slice:
+                start = i.start if i.start > 0 else None
+                stop = i.stop if i.stop > -1 else None
+                stop = None if stop == self.obj.shape[n] else stop
+                step = None if start is None and stop is None else i.step
+                l.append(slice(start, stop, step))
+            else:
+                l.append(i)
+        key = tuple(l)
+        return key
 
 
 class Location(_LocBase):
     """ class to generate .loc[] functionality """
 
     def __getitem__(self, key):
-        return self.get_idx(*self.format_key(key))
+        obj = self.get_idx(self.format_key(key))
+        if len(obj) > 1 and obj.index.iloc[:, 0].nunique() == 1:
+            idx = obj.index.iloc[:, 1:]
+            obj.index = idx
+        return obj
 
     def format_key(self, key):
-        key = (key,) if type(key) is not tuple else key
+        if (
+            type(key) is tuple
+            and len(key) > 1
+            and len(self.obj.key_labels) > 1
+            and type(key[1]) is str
+            and key[1] in self.obj.index[self.obj.key_labels[1]].unique()
+        ):
+            key = (key,)
+        else:
+            key = (key,) if type(key) is not tuple else key
         key_mask = tuple([i if i is Ellipsis else 0 for i in key])
         if len(key_mask) < len(self.obj.shape) and Ellipsis not in key_mask:
             key_mask = tuple(list(key_mask) + [Ellipsis])
-        key_mask = list(normalize_index(key_mask, self.obj.shape))
+        key_mask = list(self._normalize_index(key_mask))
         key = [item for item in key if item is not Ellipsis]
         for i in range(len(self.obj.shape)):
             if key_mask[i] == 0:
                 key_mask[i] = key[0]
                 key.pop(0)
         key = key_mask
-        # Support for multi-index
+        if type(key[0]) == pd.Series:
+            idx = key[0][key[0]].index
+        elif type(key[0]) == pd.DataFrame:
+            idx = (
+                self.obj.index.reset_index()
+                .set_index(self.obj.key_labels)
+                .loc[key[0].set_index(list(key[0].columns)).index]
+            )
+        else:
+            idx = (
+                self.obj.index.reset_index().set_index(self.obj.key_labels).loc[key[0]]
+            )
+        idx = idx.iloc[:, 0] if type(idx) is pd.DataFrame else idx
+        key[0] = _LocBase._contig_slice(idx.to_list())
         default = slice(None, None, None)
         norm = lambda k: type(k) is slice and (k.start == 0 or k == default)
-        filter_idx = None
-        if not norm(key[0]) and not type(key[0]) is pd.Series:
-            idx = self.obj.index.reset_index().set_index(self.obj.key_labels)
-            sliced = idx.loc[key[0]]
-            if type(sliced) is pd.Series:
-                sliced = sliced.to_frame().T
-                sliced.index.rename(idx.index.name, inplace=True)
-            sliced = sliced.iloc[:, 0]
-            key[0] = sliced.to_list()
-            filter_idx = list(sliced.reset_index().drop("index", 1).columns)
-            filter_idx = idx.reset_index()[filter_idx]
+
+        def normalize(key, idx):
+            mapper = {1: "columns", 2: "origin", 3: "development"}
+            out = key[idx]
+            if not norm(key[idx]) and not isinstance(key, pd.Series):
+                s = pd.Series(getattr(self.obj, mapper[idx])).to_frame().reset_index()
+                out = s.set_index(mapper[idx]).loc[key[idx]].values.flatten()
+            return out
+
+        key = [key[0]] + [normalize(key, 1), normalize(key, 2), normalize(key, 3)]
+        return key
 
         def normalize(key, idx):
             mapper = {1: "columns", 2: "origin", 3: "development"}
@@ -124,11 +169,10 @@ class Ilocation(_LocBase):
     """ class to generate .iloc[] functionality """
 
     def __getitem__(self, key):
-        return self.get_idx(normalize_index(key, self.obj.shape))
+        return self.get_idx(self._normalize_index(key))
 
     def __setitem__(self, key, values):
-        key = normalize_index(key, self.obj.shape)
-        super().__setitem__(key, values)
+        super().__setitem__(self._normalize_index(key), values)
 
 
 class TriangleSlicer:
@@ -145,6 +189,8 @@ class TriangleSlicer:
         elif key in self.key_labels:
             return self.index[key]
         else:
+            if type(key) is str and self.virtual_columns.columns.get(key, None):
+                return self.virtual_columns[key]
             key = [key] if type(key) is str else key
             idx = [list(self.vdims).index(item) for item in key]
             return self.iloc[:, idx]
@@ -152,6 +198,11 @@ class TriangleSlicer:
     def __setitem__(self, key, value):
         """ Function for pandas style column setting """
         xp = self.get_array_module()
+        if callable(value):
+            self.virtual_columns[key] = value
+            value = (self.iloc[:, 0].copy() * xp.nan).set_backend(self.array_backend)
+        else:
+            self.virtual_columns.pop(key)
         if key in self.vdims:
             i = np.where(self.vdims == key)[0][0]
             if self.array_backend == "sparse":
@@ -161,9 +212,13 @@ class TriangleSlicer:
                 value.values.coords[1] = i
                 coords = np.concatenate((before.coords, value.values.coords), axis=1)
                 data = np.concatenate((before.data, value.values.data))
-                self.values = xp(coords, data, shape=self.shape, prune=True)
+                self.values = xp(
+                    coords, data, shape=self.shape, prune=True, fill_value=xp.nan
+                )
             else:
-                self.values[:, i : i + 1] = value.values
+                if isinstance(value, TriangleSlicer):
+                    value = value.values
+                self.values[:, i : i + 1] = value
         else:
             self.vdims = self.vdims if key in self.vdims else np.append(self.vdims, key)
             try:
@@ -208,4 +263,24 @@ class TriangleSlicer:
     def _set_slicers(self):
         """ Call any time the shape of index or column changes """
         self.iloc, self.loc = Ilocation(self), Location(self)
+        self.virtual_columns = VirtualColumns(self, self.virtual_columns.columns)
         self = self._auto_sparse()
+
+
+class VirtualColumns:
+    def __init__(self, triangle, columns=None):
+        self.triangle = triangle
+        self.columns = {} if not columns else columns
+
+    def __getitem__(self, value):
+        return self.columns[value](self.triangle).rename("columns", [value])
+
+    def __setitem__(self, name, value):
+        self.columns[name] = value
+
+    def __repr__(self):
+        return str(pd.Index(self.columns.keys()))
+
+    def pop(self, key):
+        if key in list(self.columns.keys()):
+            self.columns.pop(key)
