@@ -41,7 +41,7 @@ def test_slice_by_loc_iloc():
 def test_repr():
     assert raa == raa_gt
     np.testing.assert_array_equal(
-        pd.read_html(raa._repr_html_())[0].set_index("Origin").values,
+        pd.read_html(raa._repr_html_())[0].set_index("Unnamed: 0").values,
         raa.to_frame().values,
     )
 
@@ -120,7 +120,10 @@ def test_sum_of_diff_eq_diff_of_sum():
 
 def test_append():
     assert raa == raa_gt
-    assert raa.append(raa).sum() == 2 * raa
+    raa2 = raa.copy()
+    raa2.kdims = np.array([["P2"]])
+    raa.append(raa2).sum() == raa * 2
+    assert raa.append(raa2).sum() == 2 * raa
 
 
 def test_assign_existing_col():
@@ -138,8 +141,8 @@ def test_arithmetic_across_keys():
 
 def test_grain():
     assert qtr == qtr_gt
-    actual = qtr.iloc[0, 0].grain("OYDY").values[0, 0, :, :]
-    xp = qtr.get_array_module()
+    actual = qtr.iloc[0, 0].grain("OYDY")
+    xp = actual.get_array_module()
     expected = xp.array(
         [
             [
@@ -312,7 +315,7 @@ def test_grain():
             ],
         ]
     )
-    xp.testing.assert_array_equal(actual, expected)
+    xp.testing.assert_array_equal(actual.values[0, 0, :, :], expected)
 
 
 def test_off_cycle_val_date():
@@ -373,7 +376,7 @@ def test_quantile_vs_median():
     assert tri == tri_gt
     xp = tri.get_array_module()
     xp.testing.assert_array_equal(
-        tri.quantile(0.5)["CumPaidLoss"].values, tri.median()["CumPaidLoss"].values
+        tri.quantile(q=0.5)["CumPaidLoss"].values, tri.median()["CumPaidLoss"].values
     )
 
 
@@ -407,33 +410,29 @@ def test_origin_and_value_setters():
 def test_grain_increm_arg():
     assert qtr == qtr_gt
     tri_i = qtr["incurred"].cum_to_incr()
-    np.testing.assert_array_equal(
-        tri_i.grain("OYDY").incr_to_cum(), qtr["incurred"].grain("OYDY")
-    )
+    a = tri_i.grain("OYDY").incr_to_cum()
+    assert a == qtr["incurred"].grain("OYDY").set_backend(a.array_backend)
 
 
 def test_valdev1():
     assert qtr == qtr_gt
     a = qtr.dev_to_val().val_to_dev()
     b = qtr
-    xp = a.get_array_module()
-    xp.testing.assert_array_equal(a.values, b.values)
+    assert a == b
 
 
 def test_valdev2():
     assert qtr == qtr_gt
     a = qtr.dev_to_val().grain("OYDY").val_to_dev()
     b = qtr.grain("OYDY")
-    xp = a.get_array_module()
-    xp.testing.assert_array_equal(a.values, b.values)
+    assert a == b
 
 
 def test_valdev3():
     assert qtr == qtr_gt
     a = qtr.grain("OYDY").dev_to_val().val_to_dev()
     b = qtr.grain("OYDY")
-    xp = a.get_array_module()
-    xp.testing.assert_array_equal(a.values, b.values)
+    assert a == b
 
 
 # def test_valdev4():
@@ -477,12 +476,8 @@ def test_reassignment():
 def test_dropna():
     assert tri == tri_gt
     assert tri.shape == tri.dropna().shape
-    assert tri[tri["LOB"] == "wkcomp"].iloc[-5]["CumPaidLoss"].dropna().shape == (
-        1,
-        1,
-        2,
-        2,
-    )
+    a = tri[tri["LOB"] == "wkcomp"].iloc[-5]["CumPaidLoss"].dropna().shape
+    assert a == (1, 1, 2, 2)
 
 
 def test_commutative():
@@ -500,13 +495,9 @@ def test_commutative():
     assert full.cum_to_incr().grain(
         "OYDY"
     ).val_to_dev() == full.val_to_dev().cum_to_incr().grain("OYDY")
-    assert xp.allclose(
-        xp.nan_to_num(
-            full.grain("OYDY").cum_to_incr().val_to_dev().incr_to_cum().values
-        ),
-        xp.nan_to_num(full.val_to_dev().grain("OYDY").values),
-        atol=1e-5,
-    )
+    a = full.grain("OYDY").cum_to_incr().val_to_dev().incr_to_cum()
+    b = full.val_to_dev().grain("OYDY")
+    assert abs(a - b).max().max().max() < 1e-5
 
 
 def test_broadcasting():
@@ -533,6 +524,7 @@ def test_slicers_honor_order():
         clrd.loc[clrd["LOB"] == "comauto", ["CumPaidLoss", "IncurLoss"]]
         == clrd[clrd["LOB"] == "comauto"].iloc[:, [1, 0]]
     )
+    assert clrd.groupby("LOB").sum() == clrd
 
 
 def test_exposure_tri():
@@ -573,7 +565,7 @@ def test_df_period_input():
 def test_trend_on_vector():
     assert raa == raa_gt
     d = raa.latest_diagonal
-    assert d.trend(0.05, axis=2).to_frame().astype(int).iloc[0, 0] == 29216
+    assert d.trend(0.05, axis=2).to_frame().astype(int).iloc[0, 0] == 29217
 
 
 def test_latest_diagonal_val_to_dev():
@@ -649,3 +641,180 @@ def test_loc_ellipsis():
     assert tri.loc[:, ..., 24:] == tri.loc[..., :, 24:]
     assert tri.loc[:, "CumPaidLoss"] == tri.loc[:, "CumPaidLoss", ...]
     assert tri.loc[..., "CumPaidLoss", :, :] == tri.loc[:, "CumPaidLoss", :, :]
+
+
+def missing_first_lag():
+    x = raa.copy()
+    x.values[:, :, :, 0] = 0
+    x = x.sum(0)
+    x.link_ratio.shape == (1, 1, 9, 9)
+
+
+def test_reverse_slice_integrity():
+    assert tri == tri_gt
+    assert tri.iloc[::-1, ::-1].shape == tri.shape
+    assert np.all(tri.iloc[:, ::-1].columns.values == tri.columns[::-1])
+    assert tri.iloc[tri.index.index[::-1]] + tri == 2 * tri.iloc[::-1]
+
+
+def test_loc_tuple():
+    assert tri == tri_gt
+    assert len(tri.loc[("Adriatic Ins Co", "othliab")]) == 1
+    assert tri.loc[tri.index] == tri
+
+
+def test_index_broadcasting():
+    assert tri == tri_gt
+    assert ((tri / tri.sum()) - ((1 / tri.sum()) * tri)).sum().sum().sum() < 1e-4
+
+
+def test_groupby_axis1():
+    assert tri == tri_gt
+    clrd = tri.sum("origin").sum("development")
+    groups = [i.find("Loss") >= 0 for i in clrd.columns]
+    assert np.all(
+        clrd.to_frame().groupby(groups, axis=1).sum()
+        == clrd.groupby(groups, axis=1).sum().to_frame()
+    )
+    assert np.all(
+        clrd.to_frame().groupby("LOB").sum() == clrd.groupby("LOB").sum().to_frame()
+    )
+    cl.load_sample("prism").sum().grain("OYDY")
+
+
+def test_different_forms_of_grain():
+    t = cl.load_sample("prism").sum()["Paid"]
+    assert (
+        abs(t.grain("OYDY") - t.incr_to_cum().grain("OYDY").cum_to_incr()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.grain("OYDY", trailing=True)
+            - t.incr_to_cum().grain("OYDY", trailing=True).cum_to_incr()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.incr_to_cum().grain("OYDY") - t.grain("OYDY").incr_to_cum()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.incr_to_cum().grain("OYDY", trailing=True)
+            - t.grain("OYDY", trailing=True).incr_to_cum()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.grain("OYDQ") - t.incr_to_cum().grain("OYDQ").cum_to_incr()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.grain("OYDQ", trailing=True)
+            - t.incr_to_cum().grain("OYDQ", trailing=True).cum_to_incr()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.incr_to_cum().grain("OQDM") - t.grain("OQDM").incr_to_cum()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.incr_to_cum().grain("OQDM", trailing=True)
+            - t.grain("OQDM", trailing=True).incr_to_cum()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    t = t.dev_to_val()
+    assert (
+        abs(t.grain("OYDY") - t.incr_to_cum().grain("OYDY").cum_to_incr()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.grain("OYDY", trailing=True)
+            - t.incr_to_cum().grain("OYDY", trailing=True).cum_to_incr()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.incr_to_cum().grain("OYDY") - t.grain("OYDY").incr_to_cum()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.incr_to_cum().grain("OYDY", trailing=True)
+            - t.grain("OYDY", trailing=True).incr_to_cum()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    t = t.val_to_dev()
+    t = t[t.valuation < "2017-09"]
+    assert (
+        abs(t.grain("OYDY") - t.incr_to_cum().grain("OYDY").cum_to_incr()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.grain("OYDY", trailing=True)
+            - t.incr_to_cum().grain("OYDY", trailing=True).cum_to_incr()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.incr_to_cum().grain("OYDY") - t.grain("OYDY").incr_to_cum()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.incr_to_cum().grain("OYDY", trailing=True)
+            - t.grain("OYDY", trailing=True).incr_to_cum()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    t = t.dev_to_val()
+    assert (
+        abs(t.grain("OYDY") - t.incr_to_cum().grain("OYDY").cum_to_incr()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.grain("OYDY", trailing=True)
+            - t.incr_to_cum().grain("OYDY", trailing=True).cum_to_incr()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
+    assert (
+        abs(t.incr_to_cum().grain("OYDY") - t.grain("OYDY").incr_to_cum()).sum().sum()
+        < 1e-4
+    )
+    assert (
+        abs(
+            t.incr_to_cum().grain("OYDY", trailing=True)
+            - t.grain("OYDY", trailing=True).incr_to_cum()
+        )
+        .sum()
+        .sum()
+        < 1e-4
+    )
