@@ -8,7 +8,8 @@ import copy
 import warnings
 from chainladder.core.base import TriangleBase
 from chainladder.core.correlation import DevelopmentCorrelation, ValuationCorrelation
-import datetime as dt
+from chainladder.utils.utility_functions import concat, num_to_nan
+from chainladder import ULT_VAL
 
 
 class Triangle(TriangleBase):
@@ -41,8 +42,8 @@ class Triangle(TriangleBase):
         omitted then date format will be inferred by pandas.
     cumulative : bool
         Whether the triangle is cumulative or incremental.  This attribute is
-        required to use the `grain` and `dev_to_val` methods and will be
-        automatically set when invoking `cum_to_incr` or `incr_to_cum` methods.
+        required to use the ``grain`` and ``dev_to_val`` methods and will be
+        automatically set when invoking ``cum_to_incr`` or ``incr_to_cum`` methods.
 
     Attributes
     ----------
@@ -55,7 +56,9 @@ class Triangle(TriangleBase):
     development : Series
         Represents all available levels of the development dimension.
     key_labels : list
-        Represents the `index` axis labels
+        Represents the ``index`` axis labels
+    virtual_columns : Series
+        Represents the subset of columns of the triangle that are virtual.
     valuation : DatetimeIndex
         Represents all valuation dates of each cell in the Triangle.
     origin_grain : str
@@ -63,8 +66,7 @@ class Triangle(TriangleBase):
     development_grain : str
         The grain of the development vector ('Y', 'Q', 'M')
     shape : tuple
-        The 4D shape of the triangle instance with axes corresponding to (index,
-         columns, origin, development)
+        The 4D shape of the triangle instance with axes corresponding to (index, columns, origin, development)
     link_ratio, age_to_age
         Displays age-to-age ratios for the triangle.
     valuation_date : date
@@ -91,10 +93,6 @@ class Triangle(TriangleBase):
         convertible to DataFrame.
     """
 
-    @property
-    def shape(self):
-        return self.values.shape
-
     @property
     def index(self):
         return pd.DataFrame(list(self.kdims), columns=self.key_labels)
@@ -132,7 +130,12 @@ class Triangle(TriangleBase):
 
     @property
     def origin(self):
-        return pd.DatetimeIndex(self.odims, name="origin").to_period(self.origin_grain)
+        if self.is_pattern and len(self.odims) == 1:
+            return pd.Series(["(All)"])
+        else:
+            return pd.DatetimeIndex(self.odims, name="origin").to_period(
+                self.origin_grain
+            )
 
     @origin.setter
     def origin(self, value):
@@ -144,7 +147,24 @@ class Triangle(TriangleBase):
 
     @property
     def development(self):
-        return pd.Series(list(self.ddims), name="development")
+        ddims = self.ddims.copy()
+        if self.is_val_tri:
+            formats = {"Y": "%Y", "Q": "%YQ%q", "M": "%Y-%m"}
+            ddims = ddims.to_period(freq=self.development_grain).strftime(
+                formats[self.development_grain]
+            )
+        elif self.is_pattern:
+            offset = {"Y": 12, "Q": 3, "M": 1}[self.development_grain]
+            if self.is_ultimate:
+                ddims[-1] = ddims[-2] + offset
+            if self.is_cumulative:
+                ddims = ["{}-Ult".format(ddims[i] - offset) for i in range(len(ddims))]
+            else:
+                ddims = [
+                    "{}-{}".format(ddims[i] - offset, ddims[i])
+                    for i in range(len(ddims))
+                ]
+        return pd.Series(list(ddims), name="development")
 
     @development.setter
     def development(self, value):
@@ -157,8 +177,6 @@ class Triangle(TriangleBase):
 
     @property
     def is_ultimate(self):
-        from chainladder import ULT_VAL
-
         return sum(self.valuation >= ULT_VAL[:4]) > 0
 
     @property
@@ -167,89 +185,23 @@ class Triangle(TriangleBase):
 
     @property
     def latest_diagonal(self):
-        """ The latest diagonal of the Triangle """
-        from chainladder.utils.utility_functions import num_to_nan
-
-        obj = self.copy()
-        xp = self.get_array_module()
-        val = (self.valuation == self.valuation_date).reshape(
-            self.shape[-2:], order="F"
-        )
-        val = xp.array(np.nan_to_num(val))
-        obj.values = num_to_nan(
-            xp.nansum(num_to_nan(val * 1.0) * self.values, axis=-1, keepdims=True)
-        )
-        obj.ddims = pd.DatetimeIndex(
-            [self.valuation_date], dtype="datetime64[ns]", freq=None
-        )
-        return obj
+        return self.dev_to_val().iloc[..., -1]
 
     @property
     def link_ratio(self):
-        from chainladder.utils.utility_functions import num_to_nan
-
-        xp = self.get_array_module()
-        obj = self.copy()
-        temp = num_to_nan(obj.values.copy())
-        val_array = obj.valuation.values.reshape(obj.shape[-2:], order="f")[:, 1:]
-        obj.ddims = np.array(
-            [
-                "{}-{}".format(obj.ddims[i], obj.ddims[i + 1])
-                for i in range(len(obj.ddims) - 1)
-            ]
-        )
-        obj.values = temp[..., 1:] / temp[..., :-1]
-        if self.array_backend == "sparse":
-            obj.values.shape = tuple(obj.values.coords.max(1) + 1)
-        else:
-            if xp.max(xp.sum(~xp.isnan(self.values[..., -1, :]), 2) - 1) <= 0:
-                obj.values = obj.values[..., :-1, :]
-        obj.odims = obj.odims[: obj.values.shape[2]]
+        obj = self.iloc[..., 1:] / self.iloc[..., :-1].values
+        obj = obj[obj.valuation <= obj.valuation_date]
         if hasattr(obj, "w_"):
-            if obj.shape == obj.w_[..., 0:1, : len(obj.odims), :].shape:
-                obj = obj * obj.w_[..., 0:1, : len(obj.odims), :]
+            w_ = obj.w_[..., 0:1, : len(obj.odims), :]
+            obj = obj * w_ if obj.shape == w_.shape else obj
+        obj.is_pattern = True
+        obj.is_cumulative = False
         return obj
 
     @property
     def age_to_age(self):
         return self.link_ratio
 
-    @property
-    def valuation(self):
-        from chainladder import ULT_VAL
-
-        ddims = self.ddims
-        is_val_tri = type(ddims) == pd.DatetimeIndex
-        if is_val_tri:
-            return pd.DatetimeIndex(
-                pd.DataFrame(np.repeat(self.ddims.values[None], len(self.odims), 0))
-                .unstack()
-                .values
-            )
-        if type(ddims[0]) in [np.str_, str]:
-            ddims = np.array([int(item[: item.find("-") :]) for item in ddims])
-        ddim_arr = ddims - ddims[0]
-        origin = np.minimum(self.odims, np.datetime64(self.valuation_date))
-        val_array = (
-            (origin.astype("datetime64[M]") + np.timedelta64(ddims[0], "M")).astype(
-                "datetime64[ns]"
-            )
-            - np.timedelta64(1, "ns")
-        )[:, None]
-        s = slice(None, -1) if ddims[-1] == 9999 else slice(None, None)
-        val_array = (
-            val_array.astype("datetime64[M]") + ddim_arr[s][None, :] + 1
-        ).astype("datetime64[ns]") - np.timedelta64(1, "ns")
-        if ddims[-1] == 9999:
-            val_array = np.concatenate(
-                (
-                    val_array,
-                    np.repeat(np.datetime64(ULT_VAL), val_array.shape[0])[:, None],
-                ),
-                axis=1,
-            )
-        return pd.DatetimeIndex(val_array.reshape(1, -1, order="F")[0])
-
     def incr_to_cum(self, inplace=False):
         """Method to convert an incremental triangle into a cumulative triangle.
 
@@ -262,17 +214,34 @@ class Triangle(TriangleBase):
         -------
             Updated instance of triangle accumulated along the origin
         """
-        from chainladder.utils.utility_functions import num_to_nan
-
-        xp = self.get_array_module()
         if inplace:
+            xp = self.get_array_module()
             if not self.is_cumulative:
-                self.values = (
-                    num_to_nan(xp.cumsum(xp.nan_to_num(self.values), axis=3))
-                    * self.nan_triangle[None, None, ...]
-                )
+                if self.is_pattern:
+                    values = xp.nan_to_num(self.values[..., ::-1])
+                    if self.array_backend == "sparse":
+                        xp = np
+                        values = self.set_backend("numpy").values
+                    values[values == 0] = 1.0
+                    values = xp.cumprod(values, -1)[..., ::-1]
+                    self.values = values = values * self.nan_triangle
+                    if self.array_backend == "sparse":
+                        self.values = self.get_array_module()(self.values)
+                else:
+                    if self.array_backend != "sparse":
+                        self.values = (
+                            num_to_nan(xp.cumsum(xp.nan_to_num(self.values), 3))
+                            * self.nan_triangle[None, None, ...]
+                        )
+                    else:
+                        values = xp.nan_to_num(self.values)
+                        nan_triangle = xp.nan_to_num(self.nan_triangle)
+                        l1 = lambda i: values[..., 0 : (i + 1)]
+                        l2 = lambda i: l1(i) * nan_triangle[..., i : i + 1]
+                        l3 = lambda i: l2(i).sum(3, keepdims=True)
+                        out = [l3(i) for i in range(self.shape[-1])]
+                        self.values = num_to_nan(xp.concatenate(out, axis=3))
                 self.is_cumulative = True
-                self._set_slicers()
             return self
         else:
             new_obj = self.copy()
@@ -290,26 +259,56 @@ class Triangle(TriangleBase):
         -------
             Updated instance of triangle accumulated along the origin
         """
-        xp = self.get_array_module()
-        from chainladder.utils.utility_functions import num_to_nan
-
         if inplace:
+            v = self.valuation_date
             if self.is_cumulative or self.is_cumulative is None:
-                temp = (
-                    xp.nan_to_num(self.values)[..., 1:]
-                    - xp.nan_to_num(self.values)[..., :-1]
-                )
-                temp = xp.concatenate(
-                    (xp.nan_to_num(self.values[..., 0:1]), temp), axis=3
-                )
-                self.values = num_to_nan(temp * self.nan_triangle)
+                if self.is_pattern:
+                    xp = self.get_array_module()
+                    self.values = xp.nan_to_num(self.values)
+                    self.values[self.values == 0] = 1
+                    diff = self.iloc[..., :-1] / self.iloc[..., 1:].values
+                    self = concat((diff, self.iloc[..., -1],), axis=3,)
+                    self.values = self.values * self.nan_triangle
+                else:
+                    diff = self.iloc[..., 1:] - self.iloc[..., :-1].values
+                    self = concat((self.iloc[..., 0], diff,), axis=3,)
                 self.is_cumulative = False
-                self._set_slicers()
+            self.valuation_date = v
             return self
         else:
             new_obj = self.copy()
             return new_obj.cum_to_incr(inplace=True)
 
+    def _dstep(self):
+        return {
+            "M": {"Y": 12, "Q": 3, "M": 1},
+            "Q": {"Y": 4, "Q": 1},
+            "Y": {"Y": 1},
+        }
+
+    def _val_dev(self, sign, inplace=False):
+        from chainladder import AUTO_SPARSE
+
+        backend = self.array_backend
+        obj = self.set_backend("sparse")
+        if not inplace:
+            obj.values = obj.values.copy()
+        scale = self._dstep()[obj.development_grain][obj.origin_grain]
+        offset = np.arange(obj.shape[-2]) * scale
+        offset = offset[obj.values.coords[-2]] * sign
+        obj.values.coords[-1] = obj.values.coords[-1] + offset
+        ddims = obj.valuation[obj.valuation <= obj.valuation_date]
+        ddims = len(ddims.drop_duplicates())
+        if ddims == 1 and sign == -1:
+            ddims = len(obj.odims)
+        if obj.values.density > 0 and obj.values.coords[-1].min() < 0:
+            obj.values.coords[-1] = obj.values.coords[-1] - obj.values.coords[-1].min()
+            ddims = np.max([np.max(obj.values.coords[-1]) + 1, ddims])
+        obj.values.shape = tuple(list(obj.shape[:-1]) + [ddims])
+        if AUTO_SPARSE == False or backend == "cupy":
+            obj = obj.set_backend(backend)
+        return obj
+
     def dev_to_val(self, inplace=False):
         """ Converts triangle from a development lag triangle to a valuation
         triangle.
@@ -322,83 +321,36 @@ class Triangle(TriangleBase):
 
         Returns
         -------
-            Updated instance of triangle with valuation periods.
+        Triangle
+            Updated instance of the triangle with valuation periods.
+
         """
-        if inplace:
-            if self.is_val_tri:
+        if self.is_val_tri:
+            if inplace:
                 return self
-            xp = self.get_array_module()
-            obj = self.copy()
-            if self.shape[-1] == 1:
-                return obj
-            rng = obj.valuation.unique().sort_values()
-            rng = rng[rng <= obj.valuation_date]
-            if self.is_ultimate:
-                if self.is_cumulative:
-                    obj = obj.cum_to_incr(inplace=True)
-                u_rng = rng[-1:]
-                rng = rng[:-1]
-                k, v, o = obj.shape[:-1]
-                x = xp.concatenate(
-                    (
-                        obj.values[..., :-1],
-                        xp.repeat(
-                            xp.array([[[[xp.nan]] * o] * v] * k),
-                            len(rng) - (obj.shape[-1] - 1),
-                            axis=-1,
-                        ),
-                    ),
-                    -1,
-                )
             else:
-                u_rng = []
-                x = xp.nan_to_num(obj.values)
-            val_mtrx = np.array(obj.valuation).reshape(obj.shape[-2:], order="f")[
-                :, :-1
-            ]
-            rows, column_indices = np.ogrid[: x.shape[2], : x.shape[3]]
-            r = np.arange(0, x.shape[2]) * (
-                np.where(val_mtrx[0, :] == val_mtrx[0, -1])[0][0]
-                - np.where(val_mtrx[1, :] == val_mtrx[0, -1])[0][0]
-            )
-            r[r < 0] += x.shape[1]
-            column_indices = column_indices - r[..., None]
-            if self.array_backend == "sparse":
-                b = (
-                    np.arange(x.shape[3])[None] / x.shape[3]
-                    + np.arange(x.shape[2])[:, None]
-                )[rows, column_indices]
-                c10 = (b - b % 1).astype("int64").flatten()
-                c11_new = (
-                    (np.arange(x.shape[3])[None] + np.zeros(x.shape[2])[:, None])
-                    .astype("int64")
-                    .flatten()
-                )
-                c11 = (
-                    np.round(((b - np.arange(x.shape[2])[:, None]) * x.shape[3]), 0)
-                    .astype("int64")
-                    .flatten()
-                )
-                mapper = dict(zip(list(zip(c10, c11)), c11_new))
-                x.coords[-1, :] = (
-                    pd.Series(zip(x.coords[-2:, :][0], x.coords[-2:, :][1]))
-                    .map(mapper)
-                    .values
-                )
+                return self.copy()
+        is_cumulative = self.is_cumulative
+        if self.is_full:
+            if is_cumulative:
+                obj = self.cum_to_incr(inplace=inplace)
             else:
-                x = x[..., rows, column_indices]
+                obj = self.copy()
             if self.is_ultimate:
-                x = xp.concatenate((x, obj.values[..., -1:]), -1)
-            obj.values = x
-            obj.num_to_nan()
-            obj.ddims = rng.append(u_rng)
-            obj.values = obj.values[
-                ..., : np.where(obj.ddims <= obj.valuation_date)[0].max() + 1
-            ]
-            if self.is_cumulative:
-                obj = obj.incr_to_cum(inplace=True)
-            return obj
-        return self.copy().dev_to_val(inplace=True)
+                ultimate = obj.iloc[..., -1:]
+                obj = obj.iloc[..., :-1]
+        else:
+            obj = self
+        obj = obj._val_dev(1, inplace)
+        ddims = obj.valuation[obj.valuation <= obj.valuation_date]
+        obj.ddims = ddims.drop_duplicates().sort_values()
+        if self.is_full:
+            if self.is_ultimate:
+                ultimate.ddims = pd.DatetimeIndex(ultimate.valuation[0:1])
+                obj = concat((obj, ultimate), -1)
+            if is_cumulative:
+                obj = obj.incr_to_cum(inplace=inplace)
+        return obj
 
     def val_to_dev(self, inplace=False):
         """ Converts triangle from a valuation triangle to a development lag
@@ -414,60 +366,29 @@ class Triangle(TriangleBase):
         -------
             Updated instance of triangle with development lags
         """
-        from chainladder import ULT_VAL
-
-        xp = self.get_array_module()
         if not self.is_val_tri:
-            ret_val = self
-        else:
-            if self.is_ultimate:
-                obj = self[self.valuation < ULT_VAL[:4]]
-                non_ultimates = obj.values
-                obj = obj._val_dev_chg()
-                ultimate = self[self.valuation >= ULT_VAL[:4]]
-                max_val = self[self.valuation < ULT_VAL[:4]][
-                    self.origin == self.origin.max()
-                ].valuation.max()
-                max_dev = obj[obj.origin == obj.origin.max()]
-                max_dev = max_dev[max_dev.valuation == max_val].ddims[0]
-                obj = obj[obj.development <= max_dev]
-                obj.values = xp.concatenate((obj.values, ultimate.values), axis=-1)
-                obj.ddims = np.concatenate((obj.ddims, np.array([9999])))
-                obj.valuation_date = max(obj.valuation)
-                ret_val = obj
+            if inplace:
+                return self
             else:
-                ret_val = self._val_dev_chg()
-        ret_val.values = ret_val.nan_triangle * ret_val.values
-        if inplace:
-            self = ret_val
-        return ret_val
-
-    def _val_dev_chg(self):
-        xp = self.get_array_module()
-        obj = self.copy()
-        x = xp.nan_to_num(obj.values)
-        val_mtrx = (
-            (
-                np.array(obj.valuation.year).reshape(obj.shape[-2:], order="f")
-                - np.array(pd.DatetimeIndex(obj.odims).year)[..., None]
-            )
-            * 12
-            + (
-                np.array(obj.valuation.month).reshape(obj.shape[-2:], order="f")
-                - np.array(pd.DatetimeIndex(obj.odims).month)[..., None]
-            )
-            + 1
-        )
-        rng = np.sort(np.unique(val_mtrx.flatten()[val_mtrx.flatten() > 0]))
-        val_mtrx = xp.nan_to_num(xp.array(val_mtrx))
-        # Can be slow on very large triangles
-        x = [xp.sum((val_mtrx == item) * x, -1, keepdims=True) for item in rng]
-        # End slow
-        x = xp.concatenate(x, -1)
-        obj.values = x
-        obj.ddims = np.array([item for item in rng])
-        obj.num_to_nan()
-        obj._set_slicers()
+                return self.copy()
+        if self.is_ultimate and self.shape[-1] > 1:
+            ultimate = self.iloc[..., -1:]
+            ultimate.ddims = np.array([9999])
+            obj = self.iloc[..., :-1]._val_dev(-1, inplace)
+        else:
+            obj = self.copy()._val_dev(-1, inplace)
+        val_0 = obj.valuation[0]
+        if self.ddims.shape[-1] == 1 and self.ddims[0] == self.valuation_date:
+            origin_0 = pd.to_datetime(obj.odims[-1])
+        else:
+            origin_0 = pd.to_datetime(obj.odims[0])
+        lag_0 = (val_0.year - origin_0.year) * 12 + val_0.month - origin_0.month + 1
+        scale = {"Y": 12, "Q": 3, "M": 1}[obj.development_grain]
+        obj.ddims = np.arange(obj.values.shape[-1]) * scale + lag_0
+        prune = obj[obj.origin == obj.origin.max()]
+        if self.is_ultimate and self.shape[-1] > 1:
+            obj = obj.iloc[..., : (prune.valuation <= prune.valuation_date).sum()]
+            obj = concat((obj, ultimate), -1)
         return obj
 
     def grain(self, grain="", trailing=False, inplace=False):
@@ -482,7 +403,7 @@ class Triangle(TriangleBase):
             for Origin quarter/Development Month, etc.
         trailing : bool
             For partial years/quarters, trailing will set the year/quarter end to
-            that of the latest available form the data.
+            that of the latest available from the data.
         inplace : bool
             Whether to mutate the existing Triangle instance or return a new
             one.
@@ -491,93 +412,79 @@ class Triangle(TriangleBase):
         -------
             Triangle
         """
-        ograin_new = grain[1:2]
-        ograin_old = self.origin_grain
-        dgrain_new = grain[-1]
-        dgrain_old = self.development_grain
+        ograin_old, ograin_new = self.origin_grain, grain[1:2]
+        dgrain_old, dgrain_new = self.development_grain, grain[-1]
         valid = {"Y": ["Y"], "Q": ["Q", "Y"], "M": ["Y", "Q", "M"]}
         if ograin_new not in valid.get(ograin_old, []) or dgrain_new not in valid.get(
             dgrain_old, []
         ):
             raise ValueError("New grain not compatible with existing grain")
-        if self.is_cumulative is None:
+        if (
+            self.is_cumulative is None
+            and dgrain_old != dgrain_new
+            and self.shape[-1] > 1
+        ):
             raise AttributeError(
-                """
-                The is_cumulative attribute must be set before using grain
-                method."""
+                "The is_cumulative attribute must be set before using grain method."
             )
-        if self.is_cumulative:
-            obj = self.dev_to_val(inplace=True)
+        if valid["M"].index(ograin_new) > valid["M"].index(dgrain_new):
+            raise ValueError("Origin grain must be coarser than development grain")
+        if self.is_full and not self.is_ultimate and not self.is_val_tri:
+            warnings.warn('Triangle includes extraneous development lags')
         else:
-            # Must be cumulative to work
-            obj = self.incr_to_cum().dev_to_val(inplace=True)
-        # put data in valuation mode
-        xp = self.get_array_module()
+            d_limit = None
+        obj = self.dev_to_val()
         if ograin_new != ograin_old:
-            o_dt = pd.Series(obj.odims)
             if trailing:
                 mn = self.origin[-1].strftime("%b").upper() if trailing else "DEC"
-                if ograin_new == "Q":
-                    o = np.array(
-                        pd.PeriodIndex(self.origin, freq="Q-" + mn).to_timestamp(
-                            how="s"
-                        )
-                    )
-                elif ograin_new == "Y":
-                    o = np.array(
-                        pd.PeriodIndex(self.origin, freq="A-" + mn).to_timestamp(
-                            how="s"
-                        )
-                    )
-                else:
-                    o = obj.odims
+                freq = "Q-" if ograin_new == "Q" else "A-"
+                o = pd.PeriodIndex(self.origin, freq=freq + mn)
+                o = np.array(o.to_timestamp(how="s"))
             else:
-                if ograin_new == "Q":
-                    o = np.array(
-                        pd.to_datetime(
-                            o_dt.dt.year.astype(str) + "Q" + o_dt.dt.quarter.astype(str)
-                        )
-                    )
-                elif ograin_new == "Y":
-                    o = np.array(pd.to_datetime(o_dt.dt.year, format="%Y"))
-                else:
-                    o = obj.odims
-            o_new = np.unique(o)
-            o = np.repeat(o[:, None, ...], len(o_new), axis=1)
-            o_new = np.repeat(o_new[None], len(o), axis=0)
-            o_bool = xp.repeat(xp.array(o == o_new)[:, None], len(obj.ddims), axis=1)
-            new_tri = xp.repeat(
-                xp.nan_to_num(obj.values)[..., None], o_bool.shape[-1], axis=-1
-            )
-            new_tri = xp.nansum(new_tri * o_bool, axis=2)
-            new_tri = xp.swapaxes(new_tri, -1, -2)
-            obj.values = new_tri
+                freq = "%YQ%q" if ograin_new == "Q" else "%Y"
+                o = pd.to_datetime(self.origin.strftime(freq)).values
+            values = [
+                getattr(obj.loc[..., i, :], "sum")(2, auto_sparse=False, keepdims=True)
+                for i in self.origin.groupby(o).values()
+            ]
+            obj = concat(values, axis=2, ignore_index=True)
             obj.odims = np.unique(o)
-        obj = obj.val_to_dev(inplace=True)
-        # Now do development
-        dev_grain_dict = {
-            "M": {"Y": 12, "Q": 3, "M": 1},
-            "Q": {"Y": 4, "Q": 1},
-            "Y": {"Y": 1},
-        }
-        if obj.shape[3] != 1:
-            keeps = dev_grain_dict[dgrain_old][dgrain_new]
-            keeps = np.where(np.arange(obj.shape[3]) % keeps == 0)[0]
-            keeps = -(keeps + 1)[::-1]
-            obj.values = obj.values[..., keeps]
-            obj.ddims = obj.ddims[keeps]
-        obj.origin_grain = ograin_new
-        obj.development_grain = dgrain_new
-        obj.num_to_nan()
-        if not self.is_cumulative:
-            obj = obj.cum_to_incr(inplace=True)
-        if self.is_val_tri:
-            obj = obj.dev_to_val().dropna()
+            obj.origin_grain = ograin_new
+            if len(obj.ddims) > 1 and pd.Timestamp(obj.odims[0]).strftime(
+                "%Y%m"
+            ) != obj.valuation[0].strftime("%Y%m"):
+                addl_ts = (
+                    pd.period_range(obj.odims[0], obj.valuation[0], freq="M")[:-1]
+                    .to_timestamp()
+                    .values
+                )
+                addl = obj.iloc[..., -len(addl_ts) :] * 0
+                addl.ddims = addl_ts
+                obj = concat((addl, obj), axis=-1)
+        if dgrain_old != dgrain_new and obj.shape[-1] > 1:
+            step = self._dstep()[dgrain_old][dgrain_new]
+            d = np.sort(
+                len(obj.development) - np.arange(0, len(obj.development), step) - 1
+            )
+            if obj.is_cumulative:
+                obj = obj.iloc[..., d]
+            else:
+                ddims = obj.ddims[d]
+                d2 = [d[0]] * (d[0] + 1) + list(np.repeat(np.array(d[1:]), step))
+                values = [
+                    getattr(obj.iloc[..., i], "sum")(
+                        3, auto_sparse=False, keepdims=True
+                    )
+                    for i in obj.development.groupby(d2).groups.values()
+                ]
+                obj = concat(values, axis=3, ignore_index=True)
+                obj.ddims = ddims
+            obj.development_grain = dgrain_new
+        obj = obj.dev_to_val() if self.is_val_tri else obj.val_to_dev()
         if inplace:
             self = obj
             return self
-        else:
-            return obj
+        return obj
 
     def trend(
         self,
@@ -599,14 +506,14 @@ class Triangle(TriangleBase):
         axis : str (options: ['origin', 'valuation'])
             The axis on which to apply the trend
         start: date
-            The terminal date from which trend should be calculated. If none is
+            The start date from which trend should be calculated. If none is
             provided then the latest date of the triangle is used.
         end: date
-            The terminal date to which the trend should be calculated. If none is
+            The end date to which the trend should be calculated. If none is
             provided then the earliest period of the triangle is used.
         ultimate_lag : int
-            If ultimate valuations are in the triangle, you can set the overall
-            age of the ultimate to be some lag from the latest non-Ultimate
+            If ultimate valuations are in the triangle, optionally set the overall
+            age (in months) of the ultimate to be some lag from the latest non-Ultimate
             development
 
         Returns
@@ -614,64 +521,30 @@ class Triangle(TriangleBase):
         Triangle
             updated with multiplicative trend applied.
         """
-        if kwargs.get("valuation_date", None):
-            start = kwargs["valuation_date"]
-            warnings.warn(
-                "valuation_date is deprecated, and will be removed. Use start instead."
-            )
         if axis not in ["origin", "valuation", 2, -2]:
             raise ValueError(
                 "Only origin and valuation axes are supported for trending"
             )
-
-        def val_vector(start, end, valuation):
-            if end < start:
-                val_start = np.maximum(valuation, start)
-                val_end = np.maximum(valuation, end)
-            else:
-                val_start = np.minimum(valuation, start)
-                val_end = np.minimum(valuation, end)
-            return val_start, val_end
-
         xp = self.get_array_module()
-        if not start:
-            if axis == "valuation":
-                start = np.datetime64(self.valuation_date)
-            else:
-                start = np.datetime64(np.max(self.odims))
-        else:
-            start = np.datetime64(start)
-        if not end:
-            if axis == "valuation":
-                end = np.datetime64(np.min(self.valuation))
-            else:
-                end = np.datetime64(np.min(self.odims))
-        else:
-            end = np.datetime64(end)
+        start = pd.to_datetime(start) if type(start) is str else start
+        start = self.valuation_date if start is None else start
+        end = pd.to_datetime(end) if type(end) is str else end
+        end = self.origin[0].to_timestamp() if end is None else end
         if axis in ["origin", 2, -2]:
-            val_start, val_end = val_vector(start, end, self.origin.start_time.values)
-            trend = xp.array(
-                (1 + trend) ** -(pd.Series(val_end - val_start).dt.days / 365.25)
-            )[None, None, ..., None]
-        elif axis == "valuation":
-            valuation = self.valuation
-            if self.is_ultimate and ultimate_lag is not None:
-                unit_lag = self.valuation[1] - self.valuation[0]
-                val_df = pd.DataFrame(
-                    self.valuation.values.reshape(self.shape[-2:], order="f")
-                )
-                val_df.iloc[:, -1] = val_df.iloc[:, -2] + unit_lag * ultimate_lag
-                valuation = pd.PeriodIndex(val_df.unstack().values)
-            val_start, val_end = val_vector(start, end, valuation)
-            trend = xp.array(
-                (1 + trend)
-                ** -(
-                    pd.Series(val_end - val_start).dt.days.values.reshape(
-                        self.shape[-2:], order="f"
-                    )
-                    / 365.25
-                )
-            )
+            vector = pd.DatetimeIndex(np.tile(
+                self.origin.to_timestamp(how='e').values,
+                self.shape[-1]).flatten())
+        else:
+            vector = self.valuation
+        lower, upper = (end, start) if end > start else (start, end)
+        vector = pd.DatetimeIndex(np.maximum(
+            np.minimum(np.datetime64(lower), vector.values), np.datetime64(upper)))
+        vector = ((
+                start.year - vector.year)*12+(start.month-vector.month)
+               ).values.reshape(self.shape[-2:], order="f")
+        if self.is_ultimate and ultimate_lag is not None and vector.shape[-1] > 1:
+            vector[:, -1] = vector[:, -2] + ultimate_lag
+        trend = xp.array((1 + trend) **(vector/12))[None, None, ...] * self.nan_triangle
         obj = self.copy()
         obj.values = obj.values * trend
         return obj
@@ -753,3 +626,52 @@ class Triangle(TriangleBase):
 
         """
         return ValuationCorrelation(self, p_critical, total)
+
+    def shift(self, periods=-1, axis=3):
+        """ Shift elements along an axis by desired number of periods.
+
+        Data that falls beyond the existing shape of the Triangle is eliminated
+        and new cells default to zero.
+
+        Parameters
+        ----------
+        periods : int
+            Number of periods to shift. Can be positive or negative.
+        axis : {2 or 'origin', 3 or 'development', None}, default 3
+            Shift direction.
+
+        Returns
+        -------
+        Triangle
+            updated with shifted elements
+
+        """
+        axis = self._get_axis(axis)
+        if axis < 2:
+           raise AttributeError("Lagging only supported for origin and development axes")
+        if periods == 0:
+           return self
+        if periods > 0:
+           if axis == 3:
+               out = concat((
+                   self.iloc[..., 1:].rename('development', self.development[:-1]),
+                   (self.iloc[..., -1:]*0)), axis=axis)
+           else:
+                out = concat((
+                   self.iloc[..., 1:, :].rename('origin', self.origin[:-1]),
+                   (self.iloc[..., -1:, :]*0)), axis=axis)
+        else:
+           if axis == 3:
+               out = concat((
+                   (self.iloc[..., :1]*0),
+                   self.iloc[..., :-1].rename('development', self.development[1:]),
+                   ), axis=axis)
+           else:
+                out = concat((
+                   (self.iloc[..., :1, :]*0),
+                   self.iloc[..., :-1, :].rename('origin', self.origin[1:]),
+                   ), axis=axis)
+        if abs(periods) == 1:
+           return out
+        else:
+           return self.shift(out, periods-1, axis)
