@@ -40,7 +40,7 @@ class Development(DevelopmentBase):
     fillna: float, (default=None)
         Used to fill in zero or nan values of an triangle with some non-zero
         amount.  When an link-ratio has zero as its denominator, it is automatically
-        excluded from the `ldf_` calculation.  For the specific case of 'volume'
+        excluded from the ``ldf_`` calculation.  For the specific case of 'volume'
         averaging in a deterministic method, this may be reasonable.  For all other
         averages and stochastic methods, this assumption should be avoided.
 
@@ -57,6 +57,9 @@ class Development(DevelopmentBase):
         Std_err of the ldf regression
     weight_ : pandas.DataFrame
         The weight used in the ldf regression
+    std_residuals_ : Triangle
+        A Triangle representing the weighted standardized residuals of the
+        estimator as described in Barnett and Zehnwirth.
 
     """
 
@@ -228,8 +231,10 @@ class Development(DevelopmentBase):
         ----------
         X : Triangle-like
             Set of LDFs to which the munich adjustment will be applied.
-        y : Ignored
-        sample_weight : Ignored
+        y : None
+            Ignored
+        sample_weight :
+            Ignored
 
         Returns
         -------
@@ -291,6 +296,12 @@ class Development(DevelopmentBase):
         self.ldf_ = self._param_property(X, params, 0)
         self.sigma_ = self._param_property(X, params, 1)
         self.std_err_ = self._param_property(X, params, 2)
+
+        resid = -X.iloc[..., :-1] * self.ldf_.values + X.iloc[..., 1:].values
+
+        std = xp.sqrt((1/num_to_nan(w))*(self.sigma_**2).values)
+        resid = resid/std
+        self.std_residuals_ = resid[resid.valuation < X.valuation_date]
         return self
 
     def transform(self, X):
@@ -321,5 +332,8 @@ class Development(DevelopmentBase):
         obj.values = xp.ones(obj.shape)[..., :-1] * params[..., idx : idx + 1, :]
         obj.ddims = X.link_ratio.ddims
         obj.valuation_date = pd.to_datetime(ULT_VAL)
+        obj.is_pattern = True
+        obj.is_cumulative = False
+        obj.virtual_columns.columns = {}
         obj._set_slicers()
         return obj
