@@ -59,19 +59,13 @@ class DevelopmentConstant(DevelopmentBase):
         xp = obj.get_array_module()
         obj.values = xp.ones(X.shape)[..., 0:1, :-1]
         if callable(self.patterns):
-            if len(obj.key_labels) == 1 and self.callable_axis == 0:
-                ldf = obj.index.iloc[:, 0].apply(self.patterns)
-            else:
-                ldf = obj.index.apply(self.patterns, axis=self.callable_axis).iloc[:, 0]
+            ldf = obj.index.apply(self.patterns, axis=self.callable_axis)
             ldf = (
                 pd.concat(ldf.apply(pd.DataFrame, index=[0]).values, axis=0)
                 .fillna(1)[obj.ddims[:-1]]
                 .values
             )
-            if self.callable_axis == 0:
-                ldf = xp.array(ldf[:, None, None, :])
-            else:
-                ldf = xp.array(ldf[None, :, None, :])
+            ldf = xp.array(ldf[:, None, None, :])
         else:
             ldf = xp.array([float(self.patterns[item]) for item in obj.ddims[:-1]])
             ldf = ldf[None, None, None, :]
@@ -82,9 +76,9 @@ class DevelopmentConstant(DevelopmentBase):
         obj.odims = obj.odims[0:1]
         obj._set_slicers()
         self.ldf_ = obj
+        self.ldf_.is_pattern = True
+        self.ldf_.is_cumulative = False
         self.ldf_.valuation_date = pd.to_datetime(ULT_VAL)
-        self.sigma_ = self.ldf_ * 0 + 1
-        self.std_err_ = self.ldf_ * 0 + 1
         return self
 
     def transform(self, X):
@@ -101,7 +95,7 @@ class DevelopmentConstant(DevelopmentBase):
             X_new : New triangle with transformed attributes.
         """
         X_new = X.copy()
-        triangles = ["ldf_", "sigma_", "std_err_"]
+        triangles = ["ldf_"]
         for item in triangles:
             setattr(X_new, item, getattr(self, item))
         X_new._set_slicers()
