@@ -16,9 +16,8 @@ class MunichAdjustment(DevelopmentBase):
     Parameters
     ----------
     paid_to_incurred : tuple or list of tuples
-        A dictionary representing the ``values`` of paid and incurred triangles
-        where ``values`` are an appropriate selection from :class:`Triangle`
-        ``.values``, such as ``('paid', 'incurred')``
+        A tuple representing the paid and incurred ``columns`` of the triangles
+        such as ``('paid', 'incurred')``
     fillna : boolean
         The MunichAdjustment will fail when P/I or I/P ratios cannot be calculated.
         Setting fillna to True will fill the triangle with expected amounts using
@@ -42,7 +41,7 @@ class MunichAdjustment(DevelopmentBase):
         Estimated conditional deviation around ``q_``
     lambda_ : Series or DataFrame
         Dependency coefficient between univariate chainladder link ratios and
-        `q_resids_`
+        ``q_resids_``
     ldf_ : Triangle
         The estimated bivariate loss development patterns
     cdf_ : Triangle
@@ -111,6 +110,7 @@ class MunichAdjustment(DevelopmentBase):
         self.ldf_ = self._set_ldf(
             obj, self._get_mcl_cdf(obj, self.munich_full_triangle_)
         )
+        self.ldf_.is_cumulative = False
         self.ldf_.valuation_date = pd.to_datetime(ULT_VAL)
         self._map = {
             (list(X.columns).index(x)): (num % 2, num // 2)
@@ -162,6 +162,7 @@ class MunichAdjustment(DevelopmentBase):
         for item in triangles:
             setattr(X_new, item, getattr(self, item))
         X_new._set_slicers()
+        X_new.sigma_ = X_new.std_err_ = X_new.ldf_ * 0 + 1
         return X_new
 
     def _get_p_to_i_object(self, obj):
@@ -346,6 +347,7 @@ class MunichAdjustment(DevelopmentBase):
         obj = cdf.copy()
         obj.values = ldf_tri
         obj.ddims = X.link_ratio.ddims
+        obj.is_pattern = True
         obj._set_slicers
         return obj
 
@@ -363,7 +365,8 @@ class MunichAdjustment(DevelopmentBase):
     @property
     def lambda_(self):
         obj = self.ldf_.copy()
-        obj.odims = obj.ddims = ["(All)"]
+        obj.odims = obj.odims[0:1]
+        obj.ddims = obj.ddims[0:1]
         obj.values = self._reshape("lambda_coef_")
         return obj.to_frame()
 
