@@ -2,6 +2,8 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at https://mozilla.org/MPL/2.0/.
 import numpy as np
+import pandas as pd
+import warnings
 from sklearn.base import BaseEstimator
 from chainladder.tails import TailConstant
 from chainladder.development import Development
@@ -10,6 +12,9 @@ from chainladder.core.common import Common
 
 
 class MethodBase(BaseEstimator, EstimatorIO, Common):
+
+    _estimator_type = "chainladder"
+
     def validate_X(self, X):
         obj = X.copy()
         if "ldf_" not in obj:
@@ -37,8 +42,9 @@ class MethodBase(BaseEstimator, EstimatorIO, Common):
             cdf = self.cdf_.values[list(idx.astype(int)), ..., : ultimate.shape[-1]]
         else:
             cdf = self.cdf_.values[..., : ultimate.shape[-1]]
-        a = ultimate.iloc[0, 0] * 0 + ultimate.nan_triangle
-        if a.array_backend == "sparse":
+        a = ultimate.iloc[0, 0] * 0
+        a = a + a.nan_triangle
+        if ultimate.array_backend == "sparse":
             a = a - a[a.valuation < a.valuation_date]
         a = a.set_backend(ultimate.array_backend)
         if sample_weight:
@@ -55,10 +61,13 @@ class MethodBase(BaseEstimator, EstimatorIO, Common):
 
     def _set_ult_attr(self, ultimate):
         """ Ultimate scaffolding """
+        from chainladder import ULT_VAL
+
         xp = ultimate.get_array_module()
         if ultimate.array_backend != "sparse":
             ultimate.values[~xp.isfinite(ultimate.values)] = xp.nan
-        ultimate.ddims = np.array([9999])
+        ultimate.ddims = pd.DatetimeIndex([ULT_VAL])
+        ultimate.virtual_columns.columns = {}
         ultimate._set_slicers()
         ultimate.valuation_date = ultimate.valuation.max()
         return ultimate
@@ -88,6 +97,7 @@ class MethodBase(BaseEstimator, EstimatorIO, Common):
             Returns the instance itself.
         """
         self.X_ = self.validate_X(X)
+        self.validate_weight(X, sample_weight)
         if sample_weight:
             self.sample_weight_ = sample_weight.set_backend(self.X_.array_backend)
         else:
@@ -113,6 +123,7 @@ class MethodBase(BaseEstimator, EstimatorIO, Common):
         obj = X.copy()
         xp = obj.get_array_module()
         obj.ldf_ = self.ldf_
+        self.validate_weight(X, sample_weight)
         if sample_weight:
             sample_weight = sample_weight.set_backend(obj.array_backend)
         obj.ultimate_ = self._get_ultimate(obj, sample_weight)
@@ -131,3 +142,14 @@ class MethodBase(BaseEstimator, EstimatorIO, Common):
         else:
             process_var = None
         return process_var
+
+    def validate_weight(self, X, sample_weight):
+        if (
+            sample_weight
+            and X.shape[:-1] != sample_weight.shape[:-1]
+            and sample_weight.shape[2] != 1
+            and sample_weight.shape[0] > 1
+        ):
+            warnings.warn(
+                "X and sample_weight are not aligned.  Broadcasting may occur.\n"
+            )
