@@ -57,7 +57,7 @@ class MackChainladder(Chainladder):
             Returns the instance itself.
         """
         super().fit(X, y, sample_weight)
-        if not ("average_" in self.X_ and "w_" in self.X_):
+        if "sigma_" not in self.X_:
             raise ValueError("Triangle not compatible with MackChainladder")
         # Caching full_triangle_ for fit as it is called a lot
         self.X_._full_triangle_ = self.full_triangle_
@@ -129,6 +129,7 @@ class MackChainladder(Chainladder):
                 X._full_triangle_ - X[X.valuation < X.valuation_date]
             ).iloc[:, :, :, : X.shape[3]] * X.std_err_.values
             t1_t = xp.nan_to_num(future_std_err.sum("origin").values)
+            obj.odims = obj.odims[0:1]
         else:
             nans = xp.nan_to_num(X.nan_triangle[None, None])
             nans = 1 - xp.concatenate((nans, xp.zeros((1, 1, X.shape[2], 1))), 3)
@@ -140,7 +141,7 @@ class MackChainladder(Chainladder):
         extend = X.ldf_.shape[-1] - X.shape[-1] + 1
         ldf = X.ldf_.values[..., : len(X.ddims) - 1]
         tail = X.cdf_.values[..., -extend : -extend + 1]
-        ldf = X.ldf_.get_array_module().concatenate((ldf, tail), -1)
+        ldf = xp.array(X.ldf_.get_array_module().concatenate((ldf, tail), -1))
         # Recursive Mack Formula
         for i in range(t1_t.shape[-1]):
             t1 = t1_t[..., i : i + 1] ** 2
